@@ -18,22 +18,25 @@ from mergedeep import merge
 @click.option('-r', '--replicas', default="{}")
 @click.option('-s', '--secrets', default="{}")
 @click.option('-e', '--service-params', default="{}")
+@click.option('--service-secrets', default="{}")
 @click.option('--services', default=[])
 @click.option('-s', '--show-progress', default=False, is_flag=True)
 @click.argument('instance-name', required=False)
-def apply(directory, config_file, params, tags, secrets,
-          service_params, replicas, services, instance_name, show_progress):
+def apply(directory, config_file, params, tags, secrets, service_params,
+          service_secrets, replicas, services, instance_name, show_progress):
     stackl_context = StacklContext()
     if instance_name is None:
         upload_files(directory, stackl_context)
     else:
         apply_stack_instance(config_file, params, tags, secrets,
-                             service_params, replicas, services, stackl_context,
-                             instance_name, show_progress)
+                             service_params, service_secrets, replicas,
+                             services, stackl_context, instance_name,
+                             show_progress)
 
 
 def apply_stack_instance(config_file, params, tags, secrets, service_params,
-                         replicas, services, stackl_context, instance_name, show_progress):
+                         service_secrets, replicas, services, stackl_context,
+                         instance_name, show_progress):
     final_params = {}
     for item in params:
         final_params = {**final_params, **json.loads(item)}
@@ -47,6 +50,9 @@ def apply_stack_instance(config_file, params, tags, secrets, service_params,
     service_params = json.loads(service_params)
     if "service_params" in config_doc:
         service_params = merge(config_doc['service_params'], service_params)
+    service_secrets = json.loads(service_secrets)
+    if "service_secrets" in config_doc:
+        service_secrets = merge(config_doc['service_secrets'], service_secrets)
     if "secrets" in config_doc:
         secrets = {**config_doc['secrets'], **secrets}
     if "tags" in config_doc:
@@ -61,6 +67,7 @@ def apply_stack_instance(config_file, params, tags, secrets, service_params,
         params=final_params,
         replicas=replicas,
         service_params=service_params,
+        service_secrets=service_secrets,
         secrets=secrets,
         services=services,
         tags=tags)
@@ -77,6 +84,39 @@ def apply_stack_instance(config_file, params, tags, secrets, service_params,
         show_progress_bar(stackl_context, instance_name)
 
 
+def upload_file(stackl_doc, stackl_context, path):
+    if 'name' in stackl_doc:
+        click.echo(
+            f"Applying stackl document: {str(path) + ' ' + stackl_doc['name']}"
+        )
+    else:
+        click.echo(f"Error in stackl document, no 'name' found: {path}")
+    try:
+        if stackl_doc["type"] in ["environment", "location", "zone"]:
+            stackl_context.infrastructure_base_api.put_infrastructure_base(
+                stackl_doc)
+        if stackl_doc["type"] == "functional_requirement":
+            stackl_context.functional_requirements_api.put_functional_requirement(
+                stackl_doc)
+        if stackl_doc["type"] == "service":
+            stackl_context.services_api.put_service(stackl_doc)
+        if stackl_doc["type"] == "stack_application_template":
+            stackl_context.sat_api.put_stack_application_template(stackl_doc)
+        if stackl_doc["type"] == "stack_infrastructure_template":
+            stackl_context.sit_api.put_stack_infrastructure_template(
+                stackl_doc)
+        if stackl_doc["type"] == "policy_template":
+            stackl_context.policy_templates_api.put_policy_template(stackl_doc)
+        click.echo(
+            f"Succesfully applied {stackl_doc['name']} with type {stackl_doc['type']}"
+        )
+    except stackl_client.exceptions.ApiException as e:
+        click.echo(
+            f"Failed to apply {stackl_doc['name']} with type {stackl_doc['type']}: {e.body}"
+        )
+        exit(1)
+
+
 def upload_files(directory, stackl_context):
     for path in Path(directory).rglob('*.yml'):
         with open(path, 'r') as doc:
@@ -85,36 +125,12 @@ def upload_files(directory, stackl_context):
                 continue
             click.echo(f"Reading document: {str(path)}")
             stackl_doc = yaml.load(doc.read(), Loader=yaml.FullLoader)
-            if 'name' in stackl_doc:
-                click.echo(
-                    f"Applying stackl document: {str(path) + ' ' + stackl_doc['name']}"
-                )
-            else:
-                click.echo(
-                    f"Error in stackl document, no 'name' found: {path}")
-            try:
-                if stackl_doc["type"] in ["environment", "location", "zone"]:
-                    stackl_context.infrastructure_base_api.put_infrastructure_base(
-                        stackl_doc)
-                if stackl_doc["type"] == "functional_requirement":
-                    stackl_context.functional_requirements_api.put_functional_requirement(
-                        stackl_doc)
-                if stackl_doc["type"] == "service":
-                    stackl_context.services_api.put_service(stackl_doc)
-                if stackl_doc["type"] == "stack_application_template":
-                    stackl_context.sat_api.put_stack_application_template(
-                        stackl_doc)
-                if stackl_doc["type"] == "stack_infrastructure_template":
-                    stackl_context.sit_api.put_stack_infrastructure_template(
-                        stackl_doc)
-                if stackl_doc["type"] == "policy_template":
-                    stackl_context.policy_templates_api.put_policy_template(
-                        stackl_doc)
-                click.echo(
-                    f"Succesfully applied {stackl_doc['name']} with type {stackl_doc['type']}"
-                )
-            except stackl_client.exceptions.ApiException as e:
-                click.echo(
-                    f"Failed to apply {stackl_doc['name']} with type {stackl_doc['type']}: {e.body}"
-                )
-                exit(1)
+            upload_file(stackl_doc, stackl_context, path)
+    for path in Path(directory).rglob('*.json'):
+        with open(path, 'r') as doc:
+            # ignore dotfiles
+            if path.name.startswith('.'):
+                continue
+            click.echo(f"Reading document: {str(path)}")
+            stackl_doc = yaml.load(doc.read(), Loader=yaml.FullLoader)
+            upload_file(stackl_doc, stackl_context, path)
